@ECHO OFF
REM UpdateQMgr.bat                             CACI, Inc.    2014.01     G Brown
::
:: 'UpdateQMgr' is a generalized process for defining and/or updating WMQ queue manager objects
:: as directed by WMQ MQSC files.  'UpdateQMgr' can be invoked directly by the administrator as:
::
::    UpdateQMgr [/M /C /Q /A /O] QueueManagerName
::
:: "QueueManagerName" must already exist.  Optional switches may be set to indicate which WMQ object
:: class should be processed.
::
:: There are five separately recognized classes of WMQ objects:
::     1) Queue manager attributes and WMQ objects having queue manager scope (e.g., dead letter queue)
::     2) Channels
::     3) Queues (having application domain scope)
::     4) Channel authorizations for gloabl MQE (MQ Explorer) access
::     5) Channel authorizations for queue manager server and client connections
::
:: The following switches indicate which WMQ object classes are to be processed.  Each class' associated
:: MQSC file name is given as follows:
::     /M {QueueManagerName.QMgr.mqsc}
::     /C {QueueManagerName.Channels.mqsc}
::     /Q {QueueManagerName.Queues.mqsc}
::     /A {QueueManagerName.ChlAuth.mqsc} and/or {MQE.ChlAuth.mqsc}
::
:: Additionally, there is an OAM directives class that technically is not an MQSC file:
::     /O {QueueManagerName.SetOAM.bat} and/or {MQE.SetOAM.bat}
::
:: NOTE: ONLY SPECIFY OPTIONAL SWITCHES AND A REQUIRED QUEUE MANAGER NAME IN THE COMMAND SIGNATURE!
::
:: If no switches are provided in the command signature then the default is to set all of them.
::
:: NOTE: There is a 'hidden' flag (/X) that is set when this script is invoked by 'CreateQMgr.bat'.
::       This flag indicates that certain pre-checks have already been performed by the calling script
::       and an administrator normally should not set the '/X' flag when invoking this script.
::
:: All '*.mqsc' files to be processed are expected to reside in the current directory.  Whenever a particular
:: switch is provided and its associated MQSC file is not found a warning is displayed and processing continues.
::
:: All MQSC file are first verified before being processed.  If an error is encountered the administrator
:: is instructed to review the relevant log file and make necessary corrections to the indicated MQSC file before
:: re-invoking 'UpdateQMgr'.  A specific command signature is provided such that MQSC files already successfully
:: processed are not processed again (although if they are the effect is benign).
::
::

SETLOCAL ENABLEDELAYEDEXPANSION

SET ARG=0
SET CQM=0

SET QMG=0
SET CHL=0
SET QUE=0
SET AUT=0
SET OAM=0

SET LOG=0
SET "QMGR= "
SET "SWITCH="
SET "DOMAIN="

SET QMGMQSCTAG=QMgr
SET QMGSWITCH=/M
SET CHLMQSCTAG=Channels
SET CHLSWITCH=/C
SET QUEMQSCTAG=Queues
SET QUESWITCH=/Q
SET AUTMQSCTAG=ChlAuth
SET AUTSWITCH=/A
SET OAMBATTAG=SetOAM
SET OAMSWITCH=/O

SET "MQSCTYPES=%QMGMQSCTAG% %CHLMQSCTAG% %QUEMQSCTAG% %AUTMQSCTAG% %OAMBATTAG%"
SET "MQSCOPTS="

IF "%1" == "" GOTO :USAGE


:GetSwitches
:: If there's only one arg, that's the queue manager name

IF "%2" == "" (
  SET QMGR=%1
  IF "!QMGR:~0,1!" == "/" GOTO :USAGE
  SET "MQSCRUNS=%QMG% %CHL% %QUE% %AUT% %OAM%"
  IF %CQM% == 0 GOTO :CheckWMQ
  GOTO :SetLog
)

IF /I "%1" == "/X" (
  SET CQM=1
  SHIFT /1
  GOTO :GetSwitches
)

IF /I "%1" == "/M" (
  SET QMG=1
  SET ARG=1
  SET "MQSCOPTS=%MQSCOPTS% /M"
  SHIFT /1
  GOTO :GetSwitches
)

IF /I "%1" == "/C" (
  SET CHL=1
  SET ARG=1
  SET "MQSCOPTS=%MQSCOPTS% /C"
  SHIFT /1
  GOTO :GetSwitches
)

IF /I "%1" == "/Q" (
  SET QUE=1
  SET ARG=1
  SET "MQSCOPTS=%MQSCOPTS% /Q"
  SHIFT /1
  GOTO :GetSwitches
)

IF /I "%1" == "/A" (
  SET AUT=1
  SET ARG=1
  SET "MQSCOPTS=%MQSCOPTS% /A"
  SHIFT /1
  GOTO :GetSwitches
)

IF /I "%1" == "/O" (
  SET OAM=1
  SET ARG=1
  SET "MQSCOPTS=%MQSCOPTS% /O"
  SHIFT /1
  GOTO :GetSwitches
)

ECHO.
ECHO Invalid switch "%1" in command signature.
GOTO :USAGE


:CheckWMQ
:: Check presence of WMQ and that QMgr is running

ECHO.
ECHO *** Updating WMQ Object Definitions for Queue Manager "%QMGR%" ***

:: Make sure WMQ is installed

dspmqinst > NUL

IF ERRORLEVEL 9009 (
  ECHO.
  ECHO It appears that WMQ is not installed on this host.
  ECHO If WMQ is installed be sure the WMQ bin directory is defined in the 'PATH' environment variable.
  ECHO.
  GOTO :EOF
)

:: Find where the WMQ binaries live

FOR /F "delims=" %%Z IN ('dspmqver -b -f 128') DO SET "WMQBIN=%%Z\bin"
ECHO.
ECHO Found WMQ bin Directory: "%WMQBIN%"

:: See if queue manager is defined

"%WMQBIN%\dspmq" -m %QMGR% 2>&1 | FIND "AMQ7048" > NUL

IF NOT ERRORLEVEL 1 (
  ECHO.
  ECHO Queue Manager Does Not Exist: %QMGR%
  GOTO :USAGE
)


:SetLog
::Set log file (if we've been called by the QMgr create script, continue to use that log)

IF %CQM% == 0 SET LOGFILE=%QMGR%.UpdateQMgr.log

IF %CQM% == 0 (
  ECHO %0        Update WMQ Object Definition Log File        %DATE% %TIME%
  ECHO.
) > %LOGFILE%

SET MQSCVRFY=%QMGR%.VerifyMQSC.log
SET OAMVRFY=%QMGR%.VerifyOAM.log

:: See if queue manager is running

"%WMQBIN%\dspmq" -m %QMGR% 2>&1 | FIND "Running" > NUL

IF ERRORLEVEL 1 (
  ECHO.
  ECHO Starting Queue Manager: %QMGR%

  "%WMQBIN%\strmqm" %QMGR% >> %LOGFILE% 2>&1
)

:: If no switches are provided then the default is all

IF %ARG% == 0 (
  SET QMG=1
  SET CHL=1
  SET QUE=1
  SET AUT=1
  SET OAM=1
  SET "MQSCOPTS=/M /C /Q /A /O"
)

FOR %%Z IN (%QMGMQSCTAG%.%QMG%M %CHLMQSCTAG%.%CHL%C %QUEMQSCTAG%.%QUE%Q %AUTMQSCTAG%.%AUT%A %OAMBATTAG%.%OAM%O) DO (
  SET MQSCTYPE=%%Z
  SET MQSCTYPE=!MQSCTYPE:~0,-3!
REM  ECHO MQSCTYPE: !MQSCTYPE!
  SET MQSCRUN=%%Z
  SET MQSCRUN=!MQSCRUN:~-2,-1!
REM  ECHO MQSCRUN: !MQSCRUN!
  SET SWITCH=%%Z
  SET SWITCH=/!SWITCH:~-1!
REM  ECHO SWITCH: !SWITCH!
  SET MQSCFILE=%QMGR%.!MQSCTYPE!
REM  ECHO MQSCFILE: !MQSCFILE!

  IF !MQSCRUN! == 1 (
    IF NOT !MQSCTYPE! == %OAMBATTAG% (
      SET MQSCFILE=!MQSCFILE!.mqsc
REM      ECHO MQSCFILE: !MQSCFILE!
      SET MQSCRUNOK=0

      IF EXIST !MQSCFILE! (
        CALL :VerifyMQSC !MQSCFILE!
        IF ERRORLEVEL 1 GOTO :EOF
        CALL :ExecuteMQSC !MQSCFILE!
        SET MQSCRUNOK=1
      )

      IF !MQSCTYPE! == %AUTMQSCTAG% (
        SET MQSCFILE=MQE.!MQSCTYPE!.mqsc

        IF EXIST !MQSCFILE! (
          CALL :VerifyMQSC !MQSCFILE!
          IF ERRORLEVEL 1 GOTO :EOF
          CALL :ExecuteMQSC !MQSCFILE!
          SET MQSCRUNOK=1
        )
      )

      IF !MQSCRUNOK! == 0 (
        IF !MQSCTYPE! == %AUTMQSCTAG% (
          ECHO.
          ECHO WARNING: The option switch "!SWITCH!" was specified in the command signature but a required
          ECHO          associated file "%QMGR%.!MQSCTYPE!.mqsc" and/or "MQE.!MQSCTYPE!.mqsc" was not found.
        ) ELSE (
          ECHO.
          ECHO WARNING: The option switch "!SWITCH!" was specified in the command signature but
          ECHO          a required associated file "%QMGR%.!MQSCTYPE!.mqsc" was not found.
        )
      )

      CALL :RemoveSwitch !SWITCH!

    ) ELSE (

      SET MQSCFILE=!MQSCTYPE!.bat
REM      ECHO MQSCFILE: !MQSCFILE!
      SET OAMRUNOK=0

      ECHO.
      ECHO Setting OAM Permissions on Objects for Queue Manager: %QMGR%
      ECHO.

      CALL :GetDomain

      IF EXIST !MQSCFILE! (
        CALL :RunOAM !MQSCFILE!
        IF ERRORLEVEL 1 GOTO :EOF
        SET OAMRUNOK=1
      )

      SET MQSCFILE=MQE.!MQSCTYPE!.bat

      IF EXIST !MQSCFILE! (
        CALL :RunOAM !MQSCFILE!
        IF ERRORLEVEL 1 GOTO :EOF
        SET OAMRUNOK=1
      )

      IF !OAMRUNOK! == 0 (
        ECHO.
        ECHO WARNING: The option switch "!SWITCH!" was specified in the command signature but a required
        ECHO          associated file "%QMGR%.!MQSCTYPE!.bat" and/or "MQE.!MQSCTYPE!.bat" was not found.
        ECHO.
      )

      CALL :RemoveSwitch !SWITCH!
    )
  )
)


::  Restart the queue manager

ECHO.
ECHO Restarting Queue Manager: %QMGR%

"%WMQBIN%\endmqm" -i %QMGR% >> %LOGFILE% 2>&1
"%WMQBIN%\strmqm" %QMGR% >> %LOGFILE% 2>&1

ECHO.
ECHO Carefully review the queue manager object definition update log "%LOGFILE%"
ECHO and confirm error-free execution of all WMQ and MQSC commands.
ECHO.
ECHO Successfully Updated Queue Manager: %QMGR%
GOTO :EOF


:USAGE
:: Display the command signature
ECHO.
ECHO Usage: UpdateQMgr.bat [/M /C /Q /A /O] QMgrName
ECHO.
GOTO :EOF


:VerifyMQSC

ECHO.
ECHO Verifying %1

"%WMQBIN%\runmqsc" -v %QMGR% < %1 > %MQSCVRFY% 2>&1

CALL :FindError AMQ8405 %MQSCVRFY% %1

IF ERRORLEVEL 1 (
  ECHO.
  ECHO Refer to the MQSC verify log file "%MQSCVRFY%" for error details.
  ECHO Make appropriate corrections to the "%1" file then run the following command:
  ECHO.
  ECHO      UpdateQMgr.bat !MQSCOPTS!  %QMGR%
  ECHO.
  ECHO.
  EXIT /B 1
)

ECHO Passed^^!
EXIT /B 0


:ExecuteMQSC

ECHO.
ECHO Executing %1

"%WMQBIN%\runmqsc" %QMGR% < %1 >> %LOGFILE% 2>&1

EXIT /B 0


:GetDomain

SET /P ANSWER=Were the WMQ groups defined via a Windows Domain Server [y/n]?

IF /I [!ANSWER!] == [y] (
:GetDomainName
  SET /P DOMAIN=Please enter the Windows domain to which the WMQ groups were defined:
  ECHO.
  SET /P ANSWER=Windows domain set to: !DOMAIN!  Is this correct [y/n]?

  IF /I [!ANSWER!] == [n] GOTO :GetDomainName
) ELSE (SET DOMAIN=%COMPUTERNAME%)

EXIT /B 0


:RunOAM

CALL %1 %QMGR% %DOMAIN% "" "" > %OAMVRFY% 2>&1
CALL :FindError AMQ70 %OAMVRFY% %1

IF ERRORLEVEL 1 (
  ECHO.
  ECHO Refer to the OAM verify log file "%OAMVRFY%" for error details.
  ECHO Make appropriate corrections to the "SetOAM.bat" file then run the following command:
  ECHO.
  ECHO      UpdateQMgr.bat !MQSCOPTS!  %QMGR%
  ECHO.
  ECHO.
  EXIT /B 1
)

TYPE %OAMVRFY% >> %LOGFILE%
EXIT /B 0


:FindError
FIND /I "%1" %2 > NUL
IF NOT ERRORLEVEL 1 (
  ECHO.
  ECHO ***** Error^(s^) found in file: %3 *****
  EXIT /B 1
)
EXIT /B 0


:RemoveSwitch
SET "MQSCOPTS=!MQSCOPTS:%1 =!"
EXIT /B 0

:EOF
